Imports System
Imports System.Drawing
Imports System.Windows.Forms

Imports System.IO

' single VB file WinForm application
' save as MnistDistort.vb
' to compile: >csc.exe /target:winexe MistDistort.vb

' try kernel dim = 13 k stddev = 3.0 intensity = 18.0
' image data can be found at http://yann.lecun.com/exdb/mnist/

Namespace MnistDistort

    Friend NotInheritable Class Program
        <STAThread>
        Shared Sub Main()
            Application.EnableVisualStyles()
            Application.SetCompatibleTextRenderingDefault(False)
            Application.Run(New Form1)
        End Sub
    End Class

    Partial Public Class Form1
        Inherits Form
        Private trainImages() As DigitImage = Nothing
        Private pixelFile As String = "C:\MnistDistort\train-images.idx3-ubyte" ' EDIT THIS AS NEEDED!
        Private labelFile As String = "C:\MnistDistort\train-labels.idx1-ubyte" ' EDIT THIS AS NEEDED!

        Public Sub New()
            InitializeComponent()
            textBox1.Text = pixelFile
            textBox2.Text = labelFile
            comboBox1.SelectedItem = "6" ' Magnification
            textBox3.Text = "NA" ' curr index
            textBox4.Text = "0" ' next index
            Me.ActiveControl = button1
        End Sub

        ' UI code ======================================

        Private Sub button1_Click(ByVal sender As Object, ByVal e As EventArgs) Handles button1.Click ' load MNIST train data
            Dim pixelFile As String = textBox1.Text
            Dim labelFile As String = textBox2.Text
            trainImages = LoadData(pixelFile, labelFile)
            listBox1.Items.Add("MNIST training images loaded into memory")
        End Sub

        Private Sub button2_Click(ByVal sender As Object, ByVal e As EventArgs) Handles button2.Click
            ' display 'next' image
            Dim nextIndex As Integer = Integer.Parse(textBox4.Text)
            Dim currImage As DigitImage = trainImages(nextIndex)

            Dim mag As Integer = Integer.Parse(comboBox1.SelectedItem.ToString) ' magnification
            Dim bitMap As Bitmap = MakeBitmap(currImage, mag)
            pictureBox1.Image = bitMap

            textBox3.Text = textBox4.Text ' update curr index from old next index
            textBox4.Text = (nextIndex + 1).ToString ' ++next index
            textBox8.Text = textBox3.Text ' use the curr indexs as a seed for Displacement

            listBox1.Items.Add("Curr image index = " & textBox3.Text & " label = " & currImage.label)
        End Sub

        Private Sub button3_Click(ByVal sender As Object, ByVal e As EventArgs) Handles button3.Click ' distort curr image
            Dim currIndex As Integer = Integer.Parse(textBox3.Text)
            Dim mag As Integer = Integer.Parse(comboBox1.SelectedItem.ToString)
            Dim kDim As Integer = Integer.Parse(textBox5.Text) ' Gaussian kernel dimension
            Dim kStdDev As Double = Double.Parse(textBox6.Text) ' kernel std dev
            Dim intensity As Double = Double.Parse(textBox7.Text)
            Dim rndSeed As Integer = Integer.Parse(textBox8.Text) ' use the curr image index (or manual) as seed

            Dim currImage As DigitImage = trainImages(currIndex)
            Dim distorted As DigitImage = Distort(currImage, kDim, kStdDev, intensity, rndSeed)
            Dim bitMapDist As Bitmap = MakeBitmap(distorted, mag)
            pictureBox2.Image = bitMapDist
        End Sub

        ' Logic code ===================================

        Private Shared Function LoadData(ByVal pixelFile As String, ByVal labelFile As String) As DigitImage()
            ' Load MNIST training set of 60,000 images into memory
            Dim numImages As Integer = 60000 ' ugly
            Dim result(numImages - 1) As DigitImage

            Dim pixels(27)() As Byte
            For i As Integer = 0 To pixels.Length - 1
                pixels(i) = New Byte(27) {}
            Next i

            Dim ifsPixels As New FileStream(pixelFile, FileMode.Open)
            Dim ifsLabels As New FileStream(labelFile, FileMode.Open)

            Dim brImages As New BinaryReader(ifsPixels)
            Dim brLabels As New BinaryReader(ifsLabels)

            Dim magic1 As Integer = brImages.ReadInt32 ' read and discard
            Dim imageCount As Integer = brImages.ReadInt32
            Dim numRows As Integer = brImages.ReadInt32
            Dim numCols As Integer = brImages.ReadInt32

            Dim magic2 As Integer = brLabels.ReadInt32
            Dim numLabels As Integer = brLabels.ReadInt32

            ' each image
            For di As Integer = 0 To numImages - 1
                For i As Integer = 0 To 27 ' get 28x28 pixel values
                    For j As Integer = 0 To 27
                        Dim b As Byte = brImages.ReadByte
                        pixels(i)(j) = b
                    Next j
                Next i

                Dim lbl As Byte = brLabels.ReadByte ' get the label
                Dim dImage As New DigitImage(28, 28, pixels, lbl)
                result(di) = dImage
            Next di ' each image

            ifsPixels.Close()
            brImages.Close()
            ifsLabels.Close()
            brLabels.Close()

            Return result
        End Function ' LoadData

        Private Shared Function MakeBitmap(ByVal dImage As DigitImage, ByVal mag As Integer) As Bitmap
            ' create a VB Bitmap suitable for display in a PictureBox control
            Dim width As Integer = dImage.width * mag
            Dim height As Integer = dImage.height * mag
            Dim result As New Bitmap(width, height)
            Dim gr As Graphics = Graphics.FromImage(result)
            For i As Integer = 0 To dImage.height - 1
                For j As Integer = 0 To dImage.width - 1
                    Dim pixelColor As Integer = 255 - dImage.pixels(i)(j) ' white background, black digits
                    'int pixelColor = dImage.pixels[i][j]; // black background, white digits
                    Dim c As Color = Color.FromArgb(pixelColor, pixelColor, pixelColor)
                    Dim sb As New SolidBrush(c)
                    gr.FillRectangle(sb, j * mag, i * mag, mag, mag) ' fills bitmap via Graphics object
                Next j
            Next i
            Return result
        End Function

        Private Function Distort(ByVal dImage As DigitImage, ByVal kDim As Integer, ByVal kStdDev As Double, ByVal intensity As Double, ByVal seed As Integer) As DigitImage
            Dim kernel()() As Double = MakeKernel(kDim, kStdDev)
            Dim xField()() As Double = MakeDisplace(dImage.width, dImage.height, seed, kernel, intensity)
            Dim yField()() As Double = MakeDisplace(dImage.width, dImage.height, seed + 1, kernel, intensity)
            Dim newPixels()() As Byte = Displace(dImage.pixels, xField, yField)
            Return New DigitImage(dImage.width, dImage.height, newPixels, dImage.label)
        End Function

        Private Shared Function MakeKernel(ByVal [dim] As Integer, ByVal sd As Double) As Double()()
            ' make a dim x dim continuous Gaussian kernel
            If [dim] Mod 2 = 0 Then
                Throw New Exception("kernel dim must be odd")
            End If

            Dim result([dim] - 1)() As Double
            For i As Integer = 0 To [dim] - 1
                result(i) = New Double([dim] - 1) {}
            Next i

            Dim center As Integer = [dim] / 2 ' note truncation
            Dim coef As Double = 1.0 / (2.0 * Math.PI * (sd * sd)) ' normalizes so all value sum to (close) 1.0
            Dim denom As Double = 2.0 * (sd * sd)
            Dim sum As Double = 0.0 ' for more accurate normalization
            For i As Integer = 0 To [dim] - 1
                For j As Integer = 0 To [dim] - 1
                    Dim x As Integer = Math.Abs(center - j)
                    Dim y As Integer = Math.Abs(center - i)
                    Dim num As Double = -1.0 * ((x * x) + (y * y))
                    Dim z As Double = coef * Math.Exp(num / denom)
                    result(i)(j) = z
                    sum += z
                Next j
            Next i

            For i As Integer = 0 To [dim] - 1
                For j As Integer = 0 To [dim] - 1
                    result(i)(j) = result(i)(j) / sum
                Next j
            Next i

            Return result
        End Function ' MakeKernel

        Private Shared Function MakeDisplace(ByVal width As Integer, ByVal height As Integer, ByVal seed As Integer, ByVal kernel()() As Double, ByVal intensity As Double) As Double()()
            ' first, generate a (-1, +1) displacement [in either the x- or y-direction]
            ' then apply a Gaussian kernel
            ' then multiply by an intensity factor
            Dim dField(height - 1)() As Double
            For i As Integer = 0 To dField.Length - 1
                dField(i) = New Double(width - 1) {}
            Next i

            Dim rnd As New Random(seed) ' want different displacements!

            For i As Integer = 0 To dField.Length - 1
                For j As Integer = 0 To dField(i).Length - 1
                    dField(i)(j) = 2.0 * rnd.NextDouble() - 1.0 ' (1.0 - -1.0) * rnd + -1.0
                Next j
            Next i

            dField = ApplyKernel(dField, kernel) ' smooth displacement vectors..

            For i As Integer = 0 To dField.Length - 1
                For j As Integer = 0 To dField(i).Length - 1
                    dField(i)(j) *= intensity
                Next j
            Next i

            Return dField
        End Function

        Private Shared Function ApplyKernel(ByVal dField()() As Double, ByVal kernel()() As Double) As Double()()
            ' 'convolve' displacement field with a kernel
            Dim borderAdd As Integer = kernel.Length \ 2 ' ex: for 7x7 need 3 extra rows top bottom left right
            Dim padded()() As Double = Pad(dField, borderAdd, borderAdd, borderAdd, borderAdd)

            Dim result(dField.Length - 1)() As Double
            For i As Integer = 0 To result.Length - 1
                result(i) = New Double(dField(i).Length - 1) {}
            Next i

            For i As Integer = 0 To dField.Length - 1 ' i pts to dField row
                For j As Integer = 0 To dField(i).Length - 1 ' j = dField col
                    Dim newValue As Double = 0.0 ' new value based on old val and 8 neighbors

                    Dim kr As Integer = 0
                    Dim pr As Integer = i
                    Do While kr < kernel.Length ' kr = kernel row
                        Dim kc As Integer = 0
                        Dim pc As Integer = j
                        Do While kc < kernel.Length
                            newValue += kernel(kr)(kc) * padded(pr)(pc) ' pr = padded row (dField vals)
                            kc += 1
                            pc += 1
                        Loop
                        kr += 1
                        pr += 1
                    Loop
                    result(i)(j) = newValue
                Next j
            Next i
            Return result
        End Function ' ApplyKernel

        'static double[][] ApplyKernel(double[][] matrix, double[][] kernel, bool noPadding)
        '{
        '  // matrix values 'out-of-range' are left alone
        '  int n = kernel.Length / 2; // ex: for 7x7 need 3 extra rows top bottom left right. assumes square kernel

        '  double[][] result = new double[matrix.Length][];
        '  for (int i = 0; i < result.Length; ++i)
        '    result[i] = new double[matrix[i].Length];

        '  for (int i = 0; i < matrix.Length; ++i) // i pts to matrix row
        '  {
        '    for (int j = 0; j < matrix[i].Length; ++j) // j = matrix col
        '    {
        '      if (i - n < 0 || i + n >= matrix.Length || j - n < 0 || j + n >= matrix[i].Length)
        '      {
        '        result[i][j] = matrix[i][j];
        '        continue; // bad i or j
        '      }

        '      double newValue = 0.0;

        '      for (int kr = 0, mr = i - n; kr < kernel.Length; ++kr, ++mr) // kernel row, matrix row
        '      {
        '        for (int kc = 0, mc = j - n; kc < kernel.Length; ++kc, ++mc) // kernel col
        '        {
        '          double prod = (kernel[kr][kc] * matrix[mr][mc]);
        '          newValue += prod;
        '        } // kr
        '      } // kc
        '      //result[i][j] = (int)Math.Round(newValue, 0);
        '      result[i][j] = newValue;

        '    } // j
        '  } // i
        '  return result;
        '}

        Private Shared Function Pad(ByVal dField()() As Double, ByVal top As Integer, ByVal bottom As Integer, ByVal left As Integer, ByVal right As Integer) As Double()()
            Dim oRows As Integer = dField.Length ' original number rows
            Dim oCols As Integer = dField(0).Length
            Dim nRows As Integer = oRows + top + bottom ' new (padded) number rows
            Dim nCols As Integer = oCols + left + right

            Dim result(nRows - 1)() As Double
            For i As Integer = 0 To nRows - 1
                result(i) = New Double(nCols - 1) {}
            Next i

            For i As Integer = top To oRows + top - 1
                For j As Integer = left To oCols + left - 1
                    result(i)(j) = dField(i - top)(j - left) ' copy orig values to center area
                Next j
            Next i
            Return result
        End Function ' Pad

        Private Shared Function Displace(ByVal pixels()() As Byte, ByVal xField()() As Double, ByVal yField()() As Double) As Byte()()
            ' take each pixel value and replace it with a val detemined by the x- and y- fields

            Dim resultPixels(pixels.Length - 1)() As Byte
            For i As Integer = 0 To pixels.Length - 1
                resultPixels(i) = New Byte(pixels(i).Length - 1) {}
            Next i

            'int picker = 0;
            For i As Integer = 0 To pixels.Length - 1
                For j As Integer = 0 To pixels(i).Length - 1
                    'int ii = i + (int)yField[i][j];
                    'int jj = j + (int)xField[i][j];

                    Dim lowii As Integer = i + CInt(Fix(Math.Floor(yField(i)(j))))
                    Dim hiii As Integer = i + CInt(Fix(Math.Ceiling(yField(i)(j))))
                    Dim lowjj As Integer = j + CInt(Fix(Math.Floor(xField(i)(j))))
                    Dim hijj As Integer = j + CInt(Fix(Math.Ceiling(xField(i)(j))))

                    If lowii < 0 OrElse lowjj < 0 OrElse hiii > pixels.Length - 1 OrElse hijj > pixels(i).Length - 1 Then
                        resultPixels(i)(j) = 0
                    Else
                        Dim sum As Integer = pixels(lowii)(lowjj) + pixels(lowii)(hijj) + pixels(hiii)(lowjj) + pixels(hiii)(hijj)
                        Dim avg As Double = Math.Round(sum / 4.0)
                        Dim avgVal As Byte = CByte(avg)
                        resultPixels(i)(j) = avgVal ' avg value produces a smoothing effect
                    End If
                Next j
            Next i
            Return resultPixels
        End Function

        ' ------------------ VS-generated code

        Private textBox1 As System.Windows.Forms.TextBox
        Private textBox2 As System.Windows.Forms.TextBox
        Private pictureBox1 As System.Windows.Forms.PictureBox
        Private pictureBox2 As System.Windows.Forms.PictureBox
        Private listBox1 As System.Windows.Forms.ListBox
        Private WithEvents button1 As System.Windows.Forms.Button
        Private textBox3 As System.Windows.Forms.TextBox
        Private textBox4 As System.Windows.Forms.TextBox
        Private comboBox1 As System.Windows.Forms.ComboBox
        Private WithEvents button2 As System.Windows.Forms.Button
        Private label1 As System.Windows.Forms.Label
        Private label2 As System.Windows.Forms.Label
        Private label3 As System.Windows.Forms.Label
        Private textBox5 As System.Windows.Forms.TextBox
        Private textBox6 As System.Windows.Forms.TextBox
        Private textBox7 As System.Windows.Forms.TextBox
        Private label4 As System.Windows.Forms.Label
        Private label5 As System.Windows.Forms.Label
        Private label6 As System.Windows.Forms.Label
        Private label7 As System.Windows.Forms.Label
        Private WithEvents button3 As System.Windows.Forms.Button
        Private textBox8 As System.Windows.Forms.TextBox
        Private label8 As System.Windows.Forms.Label

        Private components As System.ComponentModel.IContainer = Nothing
        Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
            If disposing AndAlso (components IsNot Nothing) Then
                components.Dispose()
            End If
            MyBase.Dispose(disposing)
        End Sub
        Private Sub InitializeComponent()
            Me.textBox1 = New System.Windows.Forms.TextBox
            Me.textBox2 = New System.Windows.Forms.TextBox
            Me.pictureBox1 = New System.Windows.Forms.PictureBox
            Me.pictureBox2 = New System.Windows.Forms.PictureBox
            Me.listBox1 = New System.Windows.Forms.ListBox
            Me.button1 = New System.Windows.Forms.Button
            Me.textBox3 = New System.Windows.Forms.TextBox
            Me.textBox4 = New System.Windows.Forms.TextBox
            Me.comboBox1 = New System.Windows.Forms.ComboBox
            Me.button2 = New System.Windows.Forms.Button
            Me.label1 = New System.Windows.Forms.Label
            Me.label2 = New System.Windows.Forms.Label
            Me.label3 = New System.Windows.Forms.Label
            Me.textBox5 = New System.Windows.Forms.TextBox
            Me.textBox6 = New System.Windows.Forms.TextBox
            Me.textBox7 = New System.Windows.Forms.TextBox
            Me.label4 = New System.Windows.Forms.Label
            Me.label5 = New System.Windows.Forms.Label
            Me.label6 = New System.Windows.Forms.Label
            Me.label7 = New System.Windows.Forms.Label
            Me.button3 = New System.Windows.Forms.Button
            Me.textBox8 = New System.Windows.Forms.TextBox
            Me.label8 = New System.Windows.Forms.Label
            CType(Me.pictureBox1, System.ComponentModel.ISupportInitialize).BeginInit()
            CType(Me.pictureBox2, System.ComponentModel.ISupportInitialize).BeginInit()
            Me.SuspendLayout()
            ' 
            ' textBox1
            ' 
            Me.textBox1.Location = New System.Drawing.Point(12, 22)
            Me.textBox1.Name = "textBox1"
            Me.textBox1.Size = New System.Drawing.Size(308, 20)
            Me.textBox1.TabIndex = 0
            ' 
            ' textBox2
            ' 
            Me.textBox2.Location = New System.Drawing.Point(12, 48)
            Me.textBox2.Name = "textBox2"
            Me.textBox2.Size = New System.Drawing.Size(308, 20)
            Me.textBox2.TabIndex = 1
            ' 
            ' pictureBox1
            ' 
            Me.pictureBox1.BackColor = System.Drawing.SystemColors.ControlDark
            Me.pictureBox1.Location = New System.Drawing.Point(12, 143)
            Me.pictureBox1.Name = "pictureBox1"
            Me.pictureBox1.Size = New System.Drawing.Size(280, 280)
            Me.pictureBox1.TabIndex = 2
            Me.pictureBox1.TabStop = False
            ' 
            ' pictureBox2
            ' 
            Me.pictureBox2.BackColor = System.Drawing.SystemColors.Desktop
            Me.pictureBox2.Location = New System.Drawing.Point(326, 143)
            Me.pictureBox2.Name = "pictureBox2"
            Me.pictureBox2.Size = New System.Drawing.Size(280, 280)
            Me.pictureBox2.TabIndex = 3
            Me.pictureBox2.TabStop = False
            ' 
            ' listBox1
            ' 
            Me.listBox1.FormattingEnabled = True
            Me.listBox1.Location = New System.Drawing.Point(12, 443)
            Me.listBox1.Name = "listBox1"
            Me.listBox1.Size = New System.Drawing.Size(280, 108)
            Me.listBox1.TabIndex = 4
            ' 
            ' button1
            ' 
            Me.button1.Location = New System.Drawing.Point(326, 32)
            Me.button1.Name = "button1"
            Me.button1.Size = New System.Drawing.Size(95, 23)
            Me.button1.TabIndex = 5
            Me.button1.Text = "Load Images"
            Me.button1.UseVisualStyleBackColor = True
            '	  Me.button1.Click += New System.EventHandler(Me.button1_Click)
            ' 
            ' textBox3
            ' 
            Me.textBox3.Enabled = False
            Me.textBox3.Location = New System.Drawing.Point(12, 91)
            Me.textBox3.Name = "textBox3"
            Me.textBox3.Size = New System.Drawing.Size(62, 20)
            Me.textBox3.TabIndex = 6
            ' 
            ' textBox4
            ' 
            Me.textBox4.Location = New System.Drawing.Point(80, 91)
            Me.textBox4.Name = "textBox4"
            Me.textBox4.Size = New System.Drawing.Size(62, 20)
            Me.textBox4.TabIndex = 7
            ' 
            ' comboBox1
            ' 
            Me.comboBox1.FormattingEnabled = True
            Me.comboBox1.Items.AddRange(New Object() {"1", "2", "3", "4", "5", "6", "7", "8", "9", "10"})
            Me.comboBox1.Location = New System.Drawing.Point(159, 91)
            Me.comboBox1.Name = "comboBox1"
            Me.comboBox1.Size = New System.Drawing.Size(62, 21)
            Me.comboBox1.TabIndex = 8
            ' 
            ' button2
            ' 
            Me.button2.Location = New System.Drawing.Point(245, 91)
            Me.button2.Name = "button2"
            Me.button2.Size = New System.Drawing.Size(75, 23)
            Me.button2.TabIndex = 9
            Me.button2.Text = "Disply Next"
            Me.button2.UseVisualStyleBackColor = True
            '	  Me.button2.Click += New System.EventHandler(Me.button2_Click)
            ' 
            ' label1
            ' 
            Me.label1.AutoSize = True
            Me.label1.Location = New System.Drawing.Point(9, 114)
            Me.label1.Name = "label1"
            Me.label1.Size = New System.Drawing.Size(55, 13)
            Me.label1.TabIndex = 10
            Me.label1.Text = "Curr Index"
            ' 
            ' label2
            ' 
            Me.label2.AutoSize = True
            Me.label2.Location = New System.Drawing.Point(77, 114)
            Me.label2.Name = "label2"
            Me.label2.Size = New System.Drawing.Size(58, 13)
            Me.label2.TabIndex = 11
            Me.label2.Text = "Next Index"
            ' 
            ' label3
            ' 
            Me.label3.AutoSize = True
            Me.label3.Location = New System.Drawing.Point(156, 115)
            Me.label3.Name = "label3"
            Me.label3.Size = New System.Drawing.Size(70, 13)
            Me.label3.TabIndex = 12
            Me.label3.Text = "Magnification"
            ' 
            ' textBox5
            ' 
            Me.textBox5.Location = New System.Drawing.Point(553, 55)
            Me.textBox5.Name = "textBox5"
            Me.textBox5.Size = New System.Drawing.Size(53, 20)
            Me.textBox5.TabIndex = 13
            Me.textBox5.Text = "13"
            ' 
            ' textBox6
            ' 
            Me.textBox6.Location = New System.Drawing.Point(553, 81)
            Me.textBox6.Name = "textBox6"
            Me.textBox6.Size = New System.Drawing.Size(53, 20)
            Me.textBox6.TabIndex = 14
            Me.textBox6.Text = "3.0"
            ' 
            ' textBox7
            ' 
            Me.textBox7.Location = New System.Drawing.Point(553, 107)
            Me.textBox7.Name = "textBox7"
            Me.textBox7.Size = New System.Drawing.Size(53, 20)
            Me.textBox7.TabIndex = 15
            Me.textBox7.Text = "18.0"
            ' 
            ' label4
            ' 
            Me.label4.AutoSize = True
            Me.label4.Location = New System.Drawing.Point(555, 9)
            Me.label4.Name = "label4"
            Me.label4.Size = New System.Drawing.Size(51, 13)
            Me.label4.TabIndex = 16
            Me.label4.Text = "Distortion"
            ' 
            ' label5
            ' 
            Me.label5.AutoSize = True
            Me.label5.Location = New System.Drawing.Point(487, 58)
            Me.label5.Name = "label5"
            Me.label5.Size = New System.Drawing.Size(60, 13)
            Me.label5.TabIndex = 17
            Me.label5.Text = "Kernel Size"
            ' 
            ' label6
            ' 
            Me.label6.AutoSize = True
            Me.label6.Location = New System.Drawing.Point(466, 84)
            Me.label6.Name = "label6"
            Me.label6.Size = New System.Drawing.Size(79, 13)
            Me.label6.TabIndex = 18
            Me.label6.Text = "Kernel Std Dev"
            ' 
            ' label7
            ' 
            Me.label7.AutoSize = True
            Me.label7.Location = New System.Drawing.Point(466, 110)
            Me.label7.Name = "label7"
            Me.label7.Size = New System.Drawing.Size(81, 13)
            Me.label7.TabIndex = 19
            Me.label7.Text = "Alpha (intensity)"
            ' 
            ' button3
            ' 
            Me.button3.Location = New System.Drawing.Point(346, 91)
            Me.button3.Name = "button3"
            Me.button3.Size = New System.Drawing.Size(75, 23)
            Me.button3.TabIndex = 20
            Me.button3.Text = "Distort Curr"
            Me.button3.UseVisualStyleBackColor = True
            '	  Me.button3.Click += New System.EventHandler(Me.button3_Click)
            ' 
            ' textBox8
            ' 
            Me.textBox8.Location = New System.Drawing.Point(553, 29)
            Me.textBox8.Name = "textBox8"
            Me.textBox8.Size = New System.Drawing.Size(53, 20)
            Me.textBox8.TabIndex = 21
            Me.textBox8.Text = "0"
            ' 
            ' label8
            ' 
            Me.label8.AutoSize = True
            Me.label8.Location = New System.Drawing.Point(471, 32)
            Me.label8.Name = "label8"
            Me.label8.Size = New System.Drawing.Size(76, 13)
            Me.label8.TabIndex = 22
            Me.label8.Text = "Displace Seed"
            ' 
            ' Form1
            ' 
            Me.AutoScaleDimensions = New System.Drawing.SizeF(6.0F, 13.0F)
            Me.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Font
            Me.ClientSize = New System.Drawing.Size(632, 570)
            Me.Controls.Add(Me.label8)
            Me.Controls.Add(Me.textBox8)
            Me.Controls.Add(Me.button3)
            Me.Controls.Add(Me.label7)
            Me.Controls.Add(Me.label6)
            Me.Controls.Add(Me.label5)
            Me.Controls.Add(Me.label4)
            Me.Controls.Add(Me.textBox7)
            Me.Controls.Add(Me.textBox6)
            Me.Controls.Add(Me.textBox5)
            Me.Controls.Add(Me.label3)
            Me.Controls.Add(Me.label2)
            Me.Controls.Add(Me.label1)
            Me.Controls.Add(Me.button2)
            Me.Controls.Add(Me.comboBox1)
            Me.Controls.Add(Me.textBox4)
            Me.Controls.Add(Me.textBox3)
            Me.Controls.Add(Me.button1)
            Me.Controls.Add(Me.listBox1)
            Me.Controls.Add(Me.pictureBox2)
            Me.Controls.Add(Me.pictureBox1)
            Me.Controls.Add(Me.textBox2)
            Me.Controls.Add(Me.textBox1)
            Me.Name = "Form1"
            Me.Text = "MNIST Distorter"
            CType(Me.pictureBox1, System.ComponentModel.ISupportInitialize).EndInit()
            CType(Me.pictureBox2, System.ComponentModel.ISupportInitialize).EndInit()
            Me.ResumeLayout(False)
            Me.PerformLayout()
        End Sub ' InitializeComponent()



        ' ------------------



    End Class ' class Form1

    Public Class DigitImage
        ' an MNIST image of a '0' thru '9' digit
        Public width As Integer ' 28
        Public height As Integer ' 28
        Public pixels()() As Byte ' 0(white) - 255(black)
        Public label As Byte ' '0' - '9'

        Public Sub New(ByVal width As Integer, ByVal height As Integer, ByVal pixels()() As Byte, ByVal label As Byte)
            Me.width = width
            Me.height = height
            Me.pixels = New Byte(height - 1)() {}
            For i As Integer = 0 To Me.pixels.Length - 1
                Me.pixels(i) = New Byte(width - 1) {}
            Next i

            For i As Integer = 0 To height - 1
                For j As Integer = 0 To width - 1
                    Me.pixels(i)(j) = pixels(i)(j)
                Next j
            Next i

            Me.label = label
        End Sub
    End Class





End Namespace ' ns

